SUBROUTINE HORVISCUV2D(C2U,C2V)
!
!           HORVISCUV2D computes contributions from the 
!           the horizontal viscosity terms of the depth integrated moment
!           equations:
!
!           d(AM2D*(d(UA*D)/dy))/dy + d(AM2D*(d(UA*D)/dx))/dx
!  and
!           d(AM2D*(d(VA*D)/dx))/dx + d(AM2D*(d(VA*D)/dy))/dy
!
!           The contributions are 
!           stored in the 2-D fields C2U and C2V before
!           exit
!
!           d(*) is the differential operator,
!           D is the dynamic depth,
!           AM2D is the vertically averaged horizontal eddy viscosity.
!
!***ROUTINES CALLED-NONE
!***END PROLOGUE
!
!   Global variables.
!
USE STATE
IMPLICIT NONE
REAL C2U(IM,JM),C2V(IM,JM)
!
!   Local variables.
!
!   XFLUX,YFLUX are used to store the viscosity fluxes 
!                     in the two space directions
!   DAM is used to store averages of AM
!
! ..
! .. Local Arrays ..
REAL :: XFLUX(IM,JM), YFLUX(IM,JM)
REAL :: FIELDD(IM,JM),DELTAI(IM,JM)
!
REAL AREA,NETFLUX
REAL DAM
INTEGER I,J
!
C2U = 0.
C2V = 0.
AREA = DX*DY
!
! Compute UA * DU 
!
XFLUX = 0.
FIELDD = 0.
DO J = 1,JM
  DO I = 1,IM
    FIELDD(I,J) = UA(I,J)*DU(I,J)
  END DO
END DO
!
!  Compute differences in FIELDD in X-direction
!
DELTAI = 0.
DO I = 2,IM
  DO J = 1,JM
    DELTAI(I,J) = FIELDD(I,J)-FIELDD(I-1,J)
  END DO
END DO
!
!$OMP PARALLEL DO PRIVATE(I,J) FIRSTPRIVATE(DX,DY)
DO J = 2,JM-1
  DO I = 1,IM-1
    IF (FSM(I,J).NE.0.) THEN
      XFLUX(I,J) = DY*AM2D(I,J)*DELTAI(I+1,J)/DX
    ENDIF
  END DO
END DO
YFLUX = 0.
!
!  Compute differences in FIELDD in Y-direction
!
DELTAI = 0.
DO I = 1,IM
  DO J = 2,JM
    DELTAI(I,J) = FIELDD(I,J)-FIELDD(I,J-1)
  END DO
END DO
!$OMP PARALLEL DO PRIVATE(I,J,DAM) FIRSTPRIVATE(DX,DY)
DO J = 1,JM-1
  DO I = 2,IM-1
    IF (DVM(I-1,J+1).NE.0. .AND. DVM(I,J+1).NE.0.) THEN
      DAM = .25*(AM2D(I,J)+AM2D(I-1,J)+AM2D(I,J+1)+AM2D(I-1,J+1))
      YFLUX(I,J) = DX*DAM*DELTAI(I,J+1)/DY
    ENDIF
  END DO
END DO
!
!   Compute the net viscosity fluxes into each ocean cell.
!
!$OMP PARALLEL DO PRIVATE(I,J,NETFLUX) FIRSTPRIVATE(AREA)
DO J = 2,JM-1
  DO I =2,IM-1
    IF (DUM(I,J).GT.0.0) THEN
        NETFLUX = XFLUX(I,J) - XFLUX(I-1,J) + &
    &             YFLUX(I,J) - YFLUX(I,J-1) 
        C2U(I,J) =   NETFLUX/AREA
    ENDIF
  END DO
END DO
!
!   Then the momentum equation in V.
!
!   Compute VA * DV
!
XFLUX = 0.
FIELDD = 0.
  DO J = 1,JM
    DO I = 1,IM
      FIELDD(I,J) = VA(I,J)*DV(I,J)
    END DO
  END DO
!
!  Compute differences in FIELDD in X-direction
!
DELTAI = 0.
DO I = 2,IM
  DO J = 1,JM
      DELTAI(I,J) = FIELDD(I,J)-FIELDD(I-1,J)
  END DO
END DO
!$OMP PARALLEL DO PRIVATE(I,J,DAM) FIRSTPRIVATE(DX,DY)
DO J = 2,JM-1
  DO I = 1,IM-1
    IF (DUM(I+1,J-1).NE.0. .AND. DUM(I+1,J).NE.0.) THEN
      DAM = .25*(AM2D(I,J)+AM2D(I,J-1)+AM2D(I+1,J)+AM2D(I+1,J-1))
      XFLUX(I,J) = DY*DAM*DELTAI(I+1,J)/DX
    ENDIF
  END DO
END DO
!
!  Compute the viscosity fluxes for the V-equation in Y-direction.
!
YFLUX = 0.
!
!  Compute differences in FIELDD in Y-direction
!
DELTAI = 0.
DO I = 1,IM
  DO J = 2,JM
      DELTAI(I,J) = FIELDD(I,J)-FIELDD(I,J-1)
  END DO
END DO
!$OMP PARALLEL DO PRIVATE(I,J) FIRSTPRIVATE(DX,DY)
DO J = 1,JM-1
  DO I = 2,IM-1
    IF (FSM(I,J).NE.0.) THEN
        YFLUX(I,J) = DX*AM2D(I,J)*DELTAI(I,J+1)/DY
    ENDIF
  END DO
END DO
!
!   Compute the net viscosity fluxes into each ocean cell.
!
!$OMP PARALLEL DO PRIVATE(I,J,NETFLUX) FIRSTPRIVATE(AREA)
DO J = 2,JM-1
  DO I =2,IM-1
    IF (DVM(I,J).GT.0.0) THEN
        NETFLUX = XFLUX(I,J) - XFLUX(I-1,J) + &
    &             YFLUX(I,J) - YFLUX(I,J-1) 
        C2V(I,J) =   NETFLUX/AREA
    ENDIF
  END DO
END DO
!
RETURN
END
